/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how a custom comparator can be implement to
 * sort a list of strings first by size and then alphabetically.
 *
 * The Comparable interface is used to define the natural ordering of
 * objects of a certain class:
 *
 * public interface Comparable<T>
 * {
 *     int compareTo(T o);
 * }
 *
 * The Comparator interface can be used in situations where the
 * natural order of objects needs to be replaced by a more specific
 * order. A much simplified version looks like this:
 *
 * public interface Comparator<T>
 * {
 *     int compare(T o1, T o2);
 * }
 *
 * The key here is that the natural order is a property of a class
 * while a comparator exists independently.
 */

import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Collection;

class S
{
    class CustomStringComparator implements Comparator<String>
    {
	public int compare(String s1, String s2)
	{
	    int l1 = s1.length();
	    int l2 = s2.length();

	    if (l1 > l2) {
		return 1;
	    } else if (l1 < l2) {
		return -1;
	    } else {
		return s1.compareTo(s2);
	    }
	}
    }

    public static void main(String[] args)
    {
	Comparator<String> c = new S().new CustomStringComparator();
	SortedSet<String> aSet = new TreeSet<String>(c);

	for (String s: args) {
	    aSet.add(s);
	}

	for (String s: aSet) {
	    System.out.print(s + " ");
	}
	System.out.println();
    }
}
